package cc.niushuai.gasweb.config;

import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.info.Contact;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.info.License;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * Spring doc 文档配置
 *
 * @author niushuai
 * @date 2022/4/6 15:50
 */
@Configuration
public class SpringDocConfig {

    @Resource
    private SpringDocProperties springDocProperties;

    @Bean
    public OpenAPI openAPI() {

        return new OpenAPI()
                .info(new Info()
                        .title(springDocProperties.getTitle())
                        .description(springDocProperties.getDescription())
                        .version(springDocProperties.getVersion())
                        .contact(
                                new Contact()
                                        .name(springDocProperties.getAuthor())
                                        .email(springDocProperties.getEmail())
                                        .url(springDocProperties.getUrl()))
                        .license(
                                new License()
                                        .name(springDocProperties.getLicenseName())
                                        .url(springDocProperties.getLicenseUrl())))
                ;
    }

}


@Data
@Component
@PropertySource(value = {"classpath:properties/springdoc.properties"}, ignoreResourceNotFound = true, encoding = "UTF-8")
@ConfigurationProperties(prefix = "springdoc")
class SpringDocProperties {

    /**
     * 标题
     */
    private String title = "OpenAPI";
    /**
     * 描述
     */
    private String description = "OpenAPI 描述信息";
    /**
     * 负责人
     */
    private String author = "admin";
    /**
     * 负责人邮箱
     */
    private String email = "admin@test.com";
    /**
     * 团队链接
     */
    private String url = "http://baidu.com";
    /**
     * 版本号
     */
    private String version = "default";
    /**
     * 证书名称
     */
    private String licenseName = "The Apache License, Version 2.0";
    /**
     * 证书链接
     */
    private String licenseUrl = "http://www.apache.org/licenses/LICENSE-2.0.html";
}

